@@ -1,5 +1,5 @@
+import logging
 import sys
-import warnings
 from collections.abc import Iterable
 from copy import deepcopy
 from pathlib import Path
@@ -18,6 +18,8 @@ import akerbp.mlpet.imputers as imputers
 import akerbp.mlpet.preprocessors as preprocessors
 import akerbp.mlpet.utilities as utilities
 
+logger = logging.getLogger(__name__)
+
 
 class Dataset(dl.DataLoader):
     """
@@ -132,7 +134,7 @@ class Dataset(dl.DataLoader):
                 for setting_name, setting in kwargs.items():
                     local = getattr(self, setting_name, None)
                     if local is not None:
-                        warnings.warn(
+                        logger.warning(
                             "This class instance already has a value set for "
                             f"{setting_name}. You are overwriting "
                             f"it's value {local} with {setting}!",
@@ -281,10 +283,10 @@ class Dataset(dl.DataLoader):
                 feature_engineering.add_gradient_features,
                 feature_engineering.add_rolling_features,
                 feature_engineering.add_sequential_features,
-                feature_engineering.add_formations_and_groups,
-                feature_engineering.add_vertical_depths,
+                feature_engineering.add_zonation_tops,
+                feature_engineering.add_trajectories,
                 feature_engineering.add_petrophysical_features,
-                feature_engineering.add_depth_trend,
+                feature_engineering.add_tophole_metadata,
                 imputers.impute_depth_trend,
                 preprocessors.set_as_nan,
                 preprocessors.remove_outliers,
@@ -300,6 +302,7 @@ class Dataset(dl.DataLoader):
                 preprocessors.remove_noise,
                 preprocessors.drop_columns,
                 preprocessors.fill_columns,
+                preprocessors.validate_bitsize_curve,
             ]
         }
         # <--------------------- INGEST INIT INPUTS -------------------------> #
@@ -315,7 +318,6 @@ class Dataset(dl.DataLoader):
             # Any keys in the provided mappings override the base mappings
             try:
                 curve_mappings = default_mappings.base_mappings["curve_mappings"]
-                curve_mappings.update()
                 self.mappings["curve_mappings"] = {
                     **curve_mappings,
                     **self.mappings["curve_mappings"],
@@ -325,40 +327,12 @@ class Dataset(dl.DataLoader):
                     "curve_mappings"
                 ]
                 pass
-            try:
-                formations_map = default_mappings.base_mappings["formations_map"]
-                self.mappings["formations_map"] = {
-                    **formations_map,
-                    **self.mappings["formations_map"],
-                }
-            except KeyError:
-                self.mappings["formations_map"] = default_mappings.base_mappings[
-                    "formations_map"
-                ]
-                pass
-            try:
-                groups_map = default_mappings.base_mappings["groups_map"]
-                self.mappings["groups_map"] = {
-                    **groups_map,
-                    **self.mappings["groups_map"],
-                }
-            except KeyError:
-                self.mappings["groups_map"] = default_mappings.base_mappings[
-                    "groups_map"
-                ]
-                pass
-            try:
-                systems_map = default_mappings.base_mappings["systems_map"]
-                self.mappings["systems_map"] = {
-                    **systems_map,
-                    **self.mappings["systems_map"],
-                }
-            except KeyError:
-                self.mappings["systems_map"] = default_mappings.base_mappings[
-                    "systems_map"
-                ]
-                pass
+
+        for key in ["members_map", "formations_map", "groups_map", "systems_map"]:
+            if key not in self.mappings:
+                self.mappings[key] = {}
         self.curve_mappings = self.mappings["curve_mappings"]
+        self.members_map = self.mappings["members_map"]
         self.formations_map = self.mappings["formations_map"]
         self.groups_map = self.mappings["groups_map"]
         self.systems_map = self.mappings["systems_map"]
@@ -491,7 +465,7 @@ class Dataset(dl.DataLoader):
         # or from the call above to standardize_curves
         diff = original_columns - self.all_curves
         if diff:
-            warnings.warn(
+            logger.warning(
                 "The following columns were passed in the preprocessing "
                 "dataframe but are not used in any of the functions defined in "
                 "the defined preprocessing pipeline. As such they will be "
@@ -631,13 +605,21 @@ class Dataset(dl.DataLoader):
                 "sequential_features": getattr(self, "sequential_features", None),
                 "shift_size": getattr(self, "shift_size", 5),
             },
-            "add_formations_and_groups": {
+            "add_zonation_tops": {
                 "id_column": self.id_column,
                 "depth_column": getattr(self, "depth_column", None),
+                "tops_df": getattr(self, "tops_df", pd.DataFrame()),
             },
-            "add_vertical_depths": {
+            "add_tophole_metadata": {
                 "id_column": self.id_column,
-                "md_column": getattr(self, "depth_column", None),
+                "tophole_columns": getattr(self, "tophole_columns", None),
+                "tophole_df": getattr(self, "tophole_df", pd.DataFrame()),
+            },
+            "add_trajectories": {
+                "id_column": self.id_column,
+                "depth_column": getattr(self, "depth_column", None),
+                "trajectory_columns": getattr(self, "trajectory_columns", None),
+                "trajectory_df": getattr(self, "trajectory_df", pd.DataFrame()),
             },
             "add_petrophysical_features": {
                 "petrophysical_features": getattr(self, "petrophysical_features", None),
@@ -647,13 +629,6 @@ class Dataset(dl.DataLoader):
                     "nan_textual_value": self.cat_filler,
                 },
             },
-            "add_depth_trend": {
-                "id_column": self.id_column,
-                "keyword_arguments": {
-                    "nan_numerical_value": self.num_filler,
-                    "nan_textual_value": self.cat_filler,
-                },
-            },
             "simple_impute": {
                 "categorical_curves": getattr(self, "categorical_curves", None),
                 "depth_column": getattr(self, "depth_column", None),
@@ -698,6 +673,7 @@ class Dataset(dl.DataLoader):
                 "columns_to_encode": getattr(
                     self, "columns_to_encode", getattr(self, "categorical_curves", None)
                 ),
+                "members_map": getattr(self, "members_map", None),
                 "formations_map": getattr(self, "formations_map", None),
                 "groups_map": getattr(self, "groups_map", None),
                 "systems_map": getattr(self, "systems_map", None),
@@ -748,6 +724,10 @@ class Dataset(dl.DataLoader):
                 "columns_to_fill": getattr(self, "columns_to_fill", None),
                 "missing_value": getattr(self, "numerical_value", None),
             },
+            "validate_bitsize_curve": {
+                "depth_column": getattr(self, "depth_column", None),
+                "id_column": getattr(self, "id_column", None),
+            },
         }
 
         # Process wells uses a bunch of lower level functions so we need to
@@ -782,8 +762,10 @@ class Dataset(dl.DataLoader):
                         if (
                             setting_name
                             in [
+                                "members_map",
                                 "formations_map",
                                 "groups_map",
+                                "systems_map",
                                 "curves_mapping",
                                 "keyword_arguments",
                             ]
@@ -857,7 +839,7 @@ class Dataset(dl.DataLoader):
                     "These curves are being filled with num_filler: "
                     f"{expected_but_missing_num_curves}"
                 )
-            warnings.warn(
+            logger.warning(
                 warning_msg,
                 stacklevel=2,
             )
