@@ -1,14 +1,14 @@
-import warnings
-from typing import Any, Dict, List, Optional
+# ruff: noqa: N802
+# ruff: noqa: N803
+import logging
+from typing import List, Optional
 
 import numpy as np
 import pandas as pd
-from cognite.client import CogniteClient
 
-import akerbp.mlpet.utilities as utilities
+from akerbp.mlpet import utilities
 
-# ruff: noqa: N802
-# ruff: noqa: N803
+logger = logging.getLogger(__name__)
 
 
 def guess_BS_from_CALI(
@@ -357,7 +357,7 @@ def calculate_diffRes(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     right = kwargs.get("right", None)
     fill_na = kwargs.get("fill_na", None)
     if left is None or right is None:
-        warnings.warn(
+        logger.warning(
             "Not possible to calculate_diffRes because the kwargs left and/or "
             "right are not provided. Returning the dataframe without any changes.",
             stacklevel=2,
@@ -376,84 +376,232 @@ def calculate_diffRes(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     return df
 
 
-def calculate_VSH(df: pd.DataFrame, **kwargs):
+def calculate_empirical_salinity_or_water_resistivity(
+    df: pd.DataFrame, **kwargs
+) -> pd.DataFrame:
     """
-    Calculates VSH by calling the deployed automatic_vsh function in CDF.
+    Calculate NaCl concentration from water resistivity or water resistivity from NaCl concentration
+    using the Bateman and Konen method (1977).
 
-    Refer to the automatic_vsh repo for more information about supported kwargs.
+    This function uses an empirical relationship to estimate either:
+    1. NaCl concentration (salinity) from formation water resistivity (Rw) and well temperature, or
+    2. Formation water resistivity (Rw) from NaCl concentration and well temperature
 
-    Args:
-        df (pd.DataFrame): pandas df to add depth trend to
+    The Bateman and Konen method:
+        Rw @ 75 deg F = 0.0123 + (3647.5 / (salinity ^ 0.955))
 
-    Keyword Args:
-        id_column (str): REQUIRED identifier for the well column in the provided dataframe
-            Defaults to None
-        keyword_arguments (dict): REQUIRED dictionary containing any kwargs for the function call.
-            Must contain 'nan_textual_value' & 'nan_numerical_value' key value
-            pairs.
-        client (CogniteClient): OPTIONAL client for querying depth trend function from CDF
-            Defaults to None
-        env (str): OPTIONAL environment where function is hosted (typically test or prod). Defaults to prod
-        version (str): OPTIONAL version of function to be called. Defaults to latest.
-        return_CI (bool): OPTIONAL whether to return confidence interval to the trends.
+    It calculates water resistivity (Rw) at 75 deg F (approx 23.889 °C) based on salinity.
+
+    To convert that water resisitivity to a different temperature T (in °C), the following adjustment is made:
+
+        Rw @ T = Rw @ 75 deg F * ((23.889 + 21.5) / (T + 21.5))
+
+    Where T is temperature in °C, Rw is water resistivity in ohm-m, and NaCl is in g/L.
+
+    Parameters:
+        rw_column_name (str, optional): The name of the column containing water resistivity values (ohm-m).
+            Provide this to calculate NaCl concentration.
+        salinity_column_name (str, optional): The name of the column containing NaCl concentration values (g/L).
+            Provide this to calculate water resistivity.
+        welltemp_column_name (str): The name of the column containing well temperature values (°C).
 
     Returns:
-        pd.DataFrame: dataframe with added depth trend and optionally its confidence intervals
+        pd.DataFrame: DataFrame with either:
+            - 'NaCl' column (salinity in g/L) if calculating from resistivity
+            - 'RW' column (water resistivity in ohm-m) if calculating from salinity
+
+    Raises:
+        ValueError: If neither or both rw_column_name and salinity_column_name are provided,
+                   or if required columns are missing from the dataframe, or if welltemp_column_name
+                   is not provided.
     """
+    rw_column_name = kwargs.get("rw_column_name", None)
+    salinity_column_name = kwargs.get("salinity_column_name", None)
+    welltemp_column_name = kwargs.get("welltemp_column_name", None)
 
-    id_column: str = kwargs.get("id_column", None)
-    client: CogniteClient = kwargs.get("client", None)
-    env: str = kwargs.get("env", "prod")
-    version: str = kwargs.get("version", None)
-    return_confidence_intervals: bool = kwargs.get("return_CI", False)
-    user_kwargs: Dict[str, Any] = kwargs.get("keyword_arguments", None)
-
-    # Validate input parameters
-    if id_column is None:
-        raise ValueError("id column (well name column) is a required kwarg!")
-    if user_kwargs is None:
+    # Input checks
+    if rw_column_name is not None and salinity_column_name is None:
+        calculate_salinity = True
+    elif rw_column_name is None and salinity_column_name is not None:
+        calculate_salinity = False
+    elif rw_column_name is None and salinity_column_name is None:
         raise ValueError(
-            "keyword_arguments is a required kwarg! Please refer to the docstring for more information."
+            "Not possible to generate NaCl or Rw as neither rw_column_name nor "
+            "salinity_column_name are provided."
         )
-    else:
-        if "nan_textual_value" not in user_kwargs:
+    elif rw_column_name is not None and salinity_column_name is not None:
+        raise ValueError(
+            "Don't know whether to calculate NaCl or Rw as both rw_column_name and "
+            "salinity_column_name are provided. Please provide only one of them."
+        )
+    if calculate_salinity:
+        if rw_column_name not in df.columns:
             raise ValueError(
-                "nan_textual_value is a required kwarg! Please refer to the docstring for more information."
+                f"Not possible to generate NaCl as {rw_column_name} is not present in dataframe."
             )
-        if "nan_numerical_value" not in user_kwargs:
-            raise ValueError(
-                "nan_numerical_value is a required kwarg! Please refer to the docstring for more information."
-            )
-
-    if "FORMATION_GROUP" not in df.columns:
-        if "FORMATION" in df.columns:
-            if "GROUP" in df.columns:
-                df["FORMATION_GROUP"] = df["FORMATION"].fillna(df["GROUP"])
-            else:
-                df["FORMATION_GROUP"] = df["FORMATION"]
-        elif "GROUP" in df.columns:
-            df["FORMATION_GROUP"] = df["GROUP"]
-        else:
-            df["FORMATION_GROUP"] = user_kwargs["nan_textual_value"]
-
-    if version is None:
-        version = "latest"
-        external_id = f"automatic_vsh-prediction-{env}"
     else:
-        external_id = f"automatic_vsh-prediction-{env}-{version}"
-    df_ = utilities.run_deployed_model(
-        df=df,
-        id_column=id_column,
-        keyword_arguments=user_kwargs,
-        cdf_external_id=external_id,
-        client=client,
+        if salinity_column_name not in df.columns:
+            raise ValueError(
+                f"Not possible to generate Rw as {salinity_column_name} is not present in dataframe."
+            )
+    if welltemp_column_name is None:
+        raise ValueError(
+            "Not possible to generate NaCl or Rw as welltemp_column_name is not provided."
+        )
+    if welltemp_column_name not in df.columns:
+        raise ValueError(
+            f"Not possible to generate NaCl or Rw as {welltemp_column_name} is not present in dataframe."
+        )
+
+    # Constants
+    bateman_konen_intercept = 0.0123
+    bateman_konen_gradient = 3647.5
+    lab_temperature = 45.39  # 23.889 + 21.5 rounded up
+    bateman_konen_power = 0.955
+
+    # Shift temperature
+    formation_temperature = df[welltemp_column_name] + 21.5
+
+    if calculate_salinity:
+        # Calculate denominator
+        denominator = (
+            df[rw_column_name] * formation_temperature / lab_temperature
+        ) - bateman_konen_intercept
+
+        # Remove values where denominator is less than or equal to 0
+        denominator = denominator.mask(denominator <= 0, np.nan)
+
+        # Calculate NaCl
+        # Convert from mg/L to g/L
+        df.loc[:, "NaCl"] = (bateman_konen_gradient / denominator) ** (
+            1 / bateman_konen_power
+        ) / 1000
+    else:
+        # Invert the formula to calculate Rw from NaCl
+        df.loc[:, "RW"] = (
+            bateman_konen_intercept
+            + (
+                bateman_konen_gradient
+                / (df[salinity_column_name] * 1000.0) ** bateman_konen_power
+            )
+        ) * (lab_temperature / formation_temperature)
+
+    return df
+
+
+def calculate_weighted_water_resistivity(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
+    """
+    Calculate the formation water resistivity (Rw) for a given NaCl concentration
+    and well temperature.
+
+    Parameters:
+        nacl_column_name (str): The name of the column containing NaCl concentration values (g/L).
+        welltemp_column_name (str): The name of the column containing well temperature values (°C).
+
+    Returns:
+        pd.DataFrame: DataFrame with Rw column (formation water resistivity - ohm-m).
+    """
+    # Required columns
+    id_column = kwargs.get("id_column", None)
+    rw_column_name = kwargs.get("rw_column_name", None)
+    tvd_column_name = kwargs.get("tvd_column_name", "TVDBML")  # m
+    zone_column_names = kwargs.get("zone_column_names", None)
+    # Required df with calibration points
+    rw_table = kwargs.get("rw_table", None)
+    # Pseudo temp kwargs
+    seabed_temperature = kwargs.get("seabed_temperature", 4)  # °C
+    pseudo_temperature_gradient = kwargs.get(
+        "pseudo_temperature_gradient", 3 / 100
+    )  # °C/m
+
+    # Input checks
+    if id_column is None:
+        raise ValueError("Not possible to generate Rw as id_column is not provided.")
+    if rw_table is None:
+        raise ValueError("Not possible to generate Rw as rw_table is not provided.")
+    if tvd_column_name not in df.columns:
+        raise ValueError(
+            f"Not possible to generate Rw as {tvd_column_name} is not present in dataframe."
+        )
+    if tvd_column_name not in rw_table.columns:
+        raise ValueError(
+            f"Not possible to generate Rw as {tvd_column_name} is not present in rw_table."
+        )
+    if zone_column_names is None:
+        raise ValueError(
+            "Not possible to generate Rw as zone_column_names are not provided."
+        )
+    if zone_column_names is not None:
+        for col in zone_column_names:
+            if col not in df.columns:
+                raise ValueError(
+                    f"Not possible to generate Rw as {col} is not present in dataframe."
+                )
+            if col not in rw_table.columns:
+                raise ValueError(
+                    f"Not possible to generate Rw as {col} is not present in rw_table."
+                )
+
+    if rw_column_name is None:
+        raise ValueError(
+            "Not possible to generate Rw as rw_column_name is not provided."
+        )
+
+    # Start by calculating pseudo temperature
+    df = df.assign(
+        PSEUDO_TEMPERATURE=seabed_temperature
+        + (df[tvd_column_name] * pseudo_temperature_gradient)
+    )
+    rw_table = rw_table.assign(
+        PSEUDO_TEMPERATURE=seabed_temperature
+        + (rw_table[tvd_column_name] * pseudo_temperature_gradient)
     )
 
-    # For akerbp.mlpet purposes (mostly as feat eng for ML Models) we want to
-    # rename VSH_AUT to VSH for better compability with preprocessing
-    df_ = df_.rename(columns={"VSH_AUT": "VSH"})
-    df_ = df_.drop(columns=["FORMATION_GROUP"], errors="ignore")
-    if not return_confidence_intervals:
-        output_curves = [c for c in df_.columns if "_P" not in c]
-        return df_[output_curves]
-    return df_
+    # Then calculate pseudo_salinity
+    rw_table = calculate_empirical_salinity_or_water_resistivity(
+        rw_table,
+        **{
+            "rw_column_name": rw_column_name,
+            "welltemp_column_name": "PSEUDO_TEMPERATURE",
+        },
+    )
+
+    # Now calculate pseudo salinity based on weighted nearest neighbours
+    for _, well_df in df.groupby(id_column):
+        well_df = well_df.assign(PSEUDO_SALINITY=np.nan)
+        for zone_column_name in zone_column_names:
+            well_df = utilities.estimate_parameter(
+                well_df,
+                rw_table,
+                id_column=id_column,
+                parameter_column_name="NaCl",
+                zone_column_name=zone_column_name,
+                coordinate_columns=kwargs.get(
+                    "coordinate_columns", ["X_TRAJECTORY", "Y_TRAJECTORY"]
+                ),
+                distance_metric=kwargs.get("distance_metric", "nan_euclidean"),
+                # Use squared euclidean distance
+                metric_params=kwargs.get("distance_metric_params", {"squared": True}),
+                weights=kwargs.get("weights", "distance"),
+                aggregation_method=kwargs.get("aggregation_method", "weighted_mean"),
+                scale_coordinate_columns=kwargs.get("scale_coordinate_columns", True),
+                remove_current_well=kwargs.get("remove_current_well", True),
+                nearest_neighbours=kwargs.get("nearest_neighbours", None),
+            )
+            well_df["PSEUDO_SALINITY"] = well_df["PSEUDO_SALINITY"].combine_first(
+                well_df["NaCl"]
+            )
+            well_df = well_df.drop(columns=["NaCl"])
+        df.loc[well_df.index, "PSEUDO_SALINITY"] = well_df["PSEUDO_SALINITY"]
+
+    # Finally calculate Rw based on pseudo salinity and pseudo temperature
+    df = calculate_empirical_salinity_or_water_resistivity(
+        df,
+        **{
+            "salinity_column_name": "PSEUDO_SALINITY",
+            "welltemp_column_name": "PSEUDO_TEMPERATURE",
+        },
+    )
+    df = df.drop(columns=["PSEUDO_TEMPERATURE", "PSEUDO_SALINITY"])
+
+    return df
