@@ -406,7 +406,7 @@ def plot_well_profile_missedpay(
 
     # Overwrite dataset from provided input
     for feature in curve_function_dataset_mapper.keys():
-        if feature in curve_function_mapper.keys():
+        if feature in curve_function_mapper:
             for function in curve_function_mapper[feature]:
                 curve_function_mapper[feature][function]["df"] = dfs[
                     curve_function_dataset_mapper[feature]
@@ -421,11 +421,11 @@ def plot_well_profile_missedpay(
 
     for i_curve, curve_name in enumerate(features_to_plot):
         # Map curve name in a standard name convension is specified in mappingsfile
-        if curve_name in default_mappings.base_mappings["curve_mappings"].keys():
+        if curve_name in default_mappings.base_mappings["curve_mappings"]:
             curve_name = default_mappings.base_mappings["curve_mappings"][curve_name]
 
         # Plot each curve
-        if curve_name in curve_function_mapper.keys():
+        if curve_name in curve_function_mapper:
             for function, parameters in curve_function_mapper[curve_name].items():
                 fig = function(fig=fig, i_curve=i_curve + 1, **parameters)
         else:
