@@ -115,6 +115,7 @@ def binary_transition_depths_fill_gaps(
         df2_shift_back.iterrows(),
         df2.iterrows(),
         df2_shift_forward.iterrows(),
+        strict=False,
     ):
         shift_end_half_interval = 0
 
@@ -1426,7 +1427,7 @@ def add_formations_and_groups(
             / 2
         )
     for label, formation_mid in zip(
-        formations_dict[tops_level + "_labels"], formation_midpoints
+        formations_dict[tops_level + "_labels"], formation_midpoints, strict=False
     ):
         if formation_mid < df[md_column].max() and formation_mid > df[md_column].min():
             # Add text with formation/group name
