@@ -19,7 +19,7 @@ the Dataset class.
 
 """
 
-import warnings
+import logging
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
 
@@ -34,6 +34,7 @@ import akerbp.mlpet.imputers as imputers
 import akerbp.mlpet.utilities as utilities
 
 # ruff: noqa: N806
+logger = logging.getLogger(__name__)
 
 
 def set_as_nan(df_: pd.DataFrame, **kwargs) -> pd.DataFrame:
@@ -60,8 +61,8 @@ def set_as_nan(df_: pd.DataFrame, **kwargs) -> pd.DataFrame:
     """
     df = df_.copy()
     # Processing inputs
-    numerical_curves: List[str] = kwargs.get("numerical_curves", None)
-    categorical_curves: List[str] = kwargs.get("categorical_curves", None)
+    numerical_curves: Optional[List[str]] = kwargs.get("numerical_curves", None)
+    categorical_curves: Optional[List[str]] = kwargs.get("categorical_curves", None)
     numerical_value = kwargs.get("numerical_value")
     categorical_value = kwargs.get("categorical_value")
     warn = kwargs.get("warn", True)
@@ -124,7 +125,7 @@ def remove_outliers(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
                 outliers = df[(df.GR < 0) | (df.GR > 250)]
                 valid_GR = df.GR.notna().sum()
                 if len(outliers) / valid_GR > th:
-                    warnings.warn(
+                    logger.warning(
                         f"GR has more than {th * 100}% of its values lower"
                         " than 0 or higher than 250. Replacing them with either 0"
                         " or 250. Note: This column name is the name after it has"
@@ -140,7 +141,7 @@ def remove_outliers(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
                     outliers = df[resistivity] > 100
                     valid_res = df[resistivity].notna().sum()
                     if outliers.sum() / valid_res > th:
-                        warnings.warn(
+                        logger.warning(
                             f"{resistivity} has more than {th * 100}% of its values higher"
                             " than 100. Note: This column names is the name after it has"
                             " been mapped using the provided mappings.yaml! So it could"
@@ -157,7 +158,7 @@ def remove_outliers(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
                 valid_NEU = df.NEU.notna().sum()
                 ratio = outliers.sum() / valid_NEU
                 if ratio > th:
-                    warnings.warn(
+                    logger.warning(
                         f"NEU has {ratio * 100}% of its values higher than 1"
                         " or lower than -0.5. The threshold for this warning is"
                         f" {th * 100}%",
@@ -169,7 +170,7 @@ def remove_outliers(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
                 outliers = df.PEF > 10
                 valid_PEF = df.PEF.notna().sum()
                 if outliers.sum() / valid_PEF > th:
-                    warnings.warn(
+                    logger.warning(
                         f"PEF has more than {th * 100}% of its values higher than 10",
                         stacklevel=2,
                     )
@@ -275,6 +276,8 @@ def fillna_with_fillers(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
 def encode_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     """
     Encodes categorical columns. Only available for:
+        - MEMBER column - categories are encoded using the members_map
+            provided in the kwargs.
         - FORMATION column - categories are encoded using the formations_map
             provided in the kwargs.
         - GROUP column - categories are encoded using the groups_map
@@ -293,6 +296,8 @@ def encode_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
         columns_to_encode (list): which columns to encode. Default to no columns being
             encoded. If no columns are passed the get_col_types utility function
             is used to determine the categorical columns
+        members_map (dict): A mapping dictionary mapping member names to
+            corresponding numbers. Defaults to an empty dictionary (ie no encoding).
         formations_map (dict): A mapping dictionary mapping formation names to
             corresponding numbers. Defaults to an empty dictionary (ie no encoding).
         groups_map (dict): A mapping dictionary mapping group names to
@@ -305,20 +310,21 @@ def encode_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     Returns:
         pd.DataFrame: dataframe with categorical columns encoded
     """
-    columns_to_encode: List[str] = kwargs.get("columns_to_encode", None)
-    formations_map: Dict[str, float] = kwargs.get("formations_map", None)
-    groups_map: Dict[str, float] = kwargs.get("groups_map", None)
-    systems_map: Dict[str, float] = kwargs.get("systems_map", None)
+    columns_to_encode: Optional[List[str]] = kwargs.get("columns_to_encode", None)
+    members_map: Optional[Dict[str, float]] = kwargs.get("members_map", None)
+    formations_map: Optional[Dict[str, float]] = kwargs.get("formations_map", None)
+    groups_map: Optional[Dict[str, float]] = kwargs.get("groups_map", None)
+    systems_map: Optional[Dict[str, float]] = kwargs.get("systems_map", None)
     missing_encoding_value: float = kwargs.get("missing_encoding_value", -1.0)
     warn = kwargs.get("warn", True)
 
     if columns_to_encode is None:
         _, columns_to_encode = utilities.get_col_types(df, warn=warn)
 
-    # Normalize before mapping
-    df = utilities.normalize_group_formation_system(
-        df, add_systems="SYSTEM" in columns_to_encode
-    )
+    if "MEMBER" in columns_to_encode and members_map is not None:
+        df["MEMBER"] = df["MEMBER"].map(members_map)
+        df["MEMBER"] = df["MEMBER"].fillna(missing_encoding_value)
+        df["MEMBER"] = df["MEMBER"].astype(float)
     if "FORMATION" in columns_to_encode and formations_map is not None:
         df["FORMATION"] = df["FORMATION"].map(formations_map)
         df["FORMATION"] = df["FORMATION"].fillna(missing_encoding_value)
@@ -357,20 +363,23 @@ def onehot_encode_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     Returns:
         pd.DataFrame: dataframe with categorical columns encoded
     """
-    columns_to_onehot_encode: List[str] = kwargs.get("columns_to_onehot_encode", None)
-    categories_to_keep: List[str] = kwargs.get("categories_to_keep", None)
+    columns_to_onehot_encode: Optional[List[str]] = kwargs.get(
+        "columns_to_onehot_encode", None
+    )
+    categories_to_keep: Optional[List[str]] = kwargs.get("categories_to_keep", None)
     if columns_to_onehot_encode is not None:
         encoded = pd.get_dummies(
             df[columns_to_onehot_encode],
             columns=columns_to_onehot_encode,
             dummy_na=False,
+            dtype=int,
         )
         cols_to_keep = (
             encoded.columns if categories_to_keep is None else categories_to_keep
         )
         for col in cols_to_keep:
             if col not in encoded.columns:
-                warnings.warn(
+                logger.warning(
                     f"{col} could not be found after one hot encoding. "
                     "Initialising it with all np.nans! Make sure to check the "
                     "categories_to_keep argument!",
@@ -380,7 +389,7 @@ def onehot_encode_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
         df = pd.concat([df, encoded[cols_to_keep]], axis=1)
         return df, {"onehot_columns": cols_to_keep}
     else:
-        warnings.warn(
+        logger.warning(
             "No columns to onehot encode provided. Doing nothing.",
             UserWarning,
             stacklevel=2,
@@ -403,9 +412,9 @@ def select_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     Returns:
         pd.DataFrame: dataframe with relevant curves
     """
-    curves_to_select: List[str] = kwargs.get("curves_to_select", None)
-    label_column: str = kwargs.get("label_column", None)
-    id_column: str = kwargs.get("id_column", None)
+    curves_to_select: Optional[List[str]] = kwargs.get("curves_to_select", None)
+    label_column: Optional[str] = kwargs.get("label_column", None)
+    id_column: Optional[str] = kwargs.get("id_column", None)
 
     if curves_to_select is not None:
         curves_to_keep = list(set(curves_to_select))
@@ -432,7 +441,7 @@ def drop_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     Returns:
         pd.DataFrame: dataframe with requested curves dropped
     """
-    curves_to_drop: List[str] = kwargs.get("curves_to_drop", None)
+    curves_to_drop: Optional[List[str]] = kwargs.get("curves_to_drop", None)
 
     if curves_to_drop is not None:
         curves_to_drop = list(set(curves_to_drop))
@@ -455,8 +464,10 @@ def fill_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     Returns:
         pd.DataFrame: dataframe with columns filled according to the provided mapping
     """
-    columns_to_fill: Dict[str, List[str]] = kwargs.get("columns_to_fill", None)
-    missing_value: float = kwargs.get("missing_value", None)
+    columns_to_fill: Optional[Dict[str, List[str]]] = kwargs.get(
+        "columns_to_fill", None
+    )
+    missing_value: Optional[float] = kwargs.get("missing_value", None)
 
     if missing_value is None:
         missing_value = np.nan
@@ -471,6 +482,132 @@ def fill_columns(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
     return df
 
 
+def validate_bitsize_curve(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
+    """
+    Validates the bitsize (BS) curve by flagging unusually short intervals.
+
+    Any interval where the bitsize value persists for less than the provided
+    minimum length threshold is considered invalid. All samples belonging to
+    such intervals are set to ``np.nan`` and a warning is issued.
+
+    Args:
+        df (pd.DataFrame): dataframe containing the bitsize curve.
+
+    Keyword Args:
+        bitsize_column (str, optional): Name of the bitsize curve to validate.
+            Defaults to "BS".
+        depth_column (str): Name of the measured depth column used to determine
+            interval lengths.
+        min_bit_section_length (float): Minimum acceptable interval length in
+            metres. Intervals shorter than this value are replaced with NaNs.
+        id_column (str, optional): Identifier column used to run the validation
+            per well. If not provided, the entire dataframe is treated as a
+            single well.
+
+    Returns:
+        pd.DataFrame: dataframe with invalid bitsize intervals replaced by NaNs.
+    """
+    bitsize_column: str = kwargs.get("bitsize_column", "BS")
+    depth_column: Optional[str] = kwargs.get("depth_column", None)
+    id_column: Optional[str] = kwargs.get("id_column", None)
+    min_bit_section_length: Optional[float] = kwargs.get("min_bit_section_length", None)
+    max_number_of_bitsizes_per_well: float = kwargs.get(
+        "max_number_of_bitsizes_per_well", 10
+    )
+
+    if min_bit_section_length is None:
+        raise ValueError(
+            "The 'min_bit_section_length' kwarg is required to validate the bitsize curve."
+        )
+    try:
+        min_bit_section_length = float(min_bit_section_length)
+    except (TypeError, ValueError) as exc:
+        raise ValueError(
+            "The 'min_bit_section_length' kwarg must be a numeric value expressed in metres."
+        ) from exc
+    if min_bit_section_length < 0:
+        raise ValueError("The 'min_bit_section_length' kwarg must be non-negative.")
+    if depth_column is None:
+        raise ValueError(
+            "The 'depth_column' kwarg is required to validate the bitsize curve."
+        )
+    if id_column is None:
+        raise ValueError(
+            "The 'id_column' kwarg is required to validate the bitsize curve."
+        )
+    if bitsize_column not in df.columns:
+        raise ValueError(
+            f"Bitsize column {bitsize_column} was requested for validation but is not present in the dataframe."
+        )
+    if depth_column not in df.columns:
+        raise ValueError(
+            f"Depth column {depth_column} was requested for bitsize validation but could not be found in the dataframe."
+        )
+    if id_column not in df.columns:
+        raise ValueError(
+            f"ID column {id_column} was requested for bitsize validation but could not be found in the dataframe."
+        )
+
+    if df[bitsize_column].nunique() > max_number_of_bitsizes_per_well:
+        logger.warning(
+            f"The bitsize column '{bitsize_column}' contains more than "
+            f"{max_number_of_bitsizes_per_well} unique values. The entire column ",
+            "will be set to NaN.",
+            stacklevel=2,
+        )
+        df.loc[:, bitsize_column] = np.nan
+        return df
+
+    grouped = df.groupby(id_column, dropna=True, sort=False)
+    for well_id, group in grouped:
+        if group.empty or group[bitsize_column].dropna().empty:
+            continue
+
+        work_df = group.sort_values(depth_column)
+        work_df["__is_new_run"] = work_df[bitsize_column].ne(
+            work_df[bitsize_column].shift()
+        )
+        work_df["__run_id"] = work_df["__is_new_run"].cumsum()
+
+        run_starts = work_df.loc[
+            work_df["__is_new_run"], [depth_column, bitsize_column, "__run_id"]
+        ]
+        if run_starts.empty:
+            continue
+
+        run_starts.loc[:, "end_depth"] = run_starts[depth_column].shift(-1)
+        run_starts.loc[run_starts.index[-1], "end_depth"] = work_df[depth_column].iloc[
+            -1
+        ]
+
+        run_starts.loc[:, "interval_length"] = (
+            run_starts["end_depth"] - run_starts[depth_column]
+        ).clip(lower=0)
+
+        invalid_runs = run_starts.loc[
+            run_starts[bitsize_column].notna()
+            & (run_starts["interval_length"] < min_bit_section_length)
+        ]
+        if invalid_runs.empty:
+            continue
+
+        run_ids = invalid_runs["__run_id"].tolist()
+        indices_to_null = work_df.loc[work_df["__run_id"].isin(run_ids)].index
+        df.loc[indices_to_null, bitsize_column] = np.nan
+
+        well_label = f"well '{well_id}'" if well_id is not None else "dataset"
+        logger.warning(
+            "Detected %d bitsize intervals shorter than %.2f m for %s. Setting %d samples to NaN.",
+            len(run_ids),
+            float(min_bit_section_length),
+            well_label,
+            len(indices_to_null),
+            stacklevel=2,
+        )
+
+    return df
+
+
 def normalize_curves(
     df: pd.DataFrame, **kwargs
 ) -> Union[pd.DataFrame, Tuple[pd.DataFrame, Dict[str, Any]]]:
@@ -508,11 +645,11 @@ def normalize_curves(
         tuple(pd.DataFrame, dict): pd.DataFrame with normalized values and dictionary with key wells
             that were used to normalize the curves_to_normalize
     """
-    curves_to_normalize: List[str] = kwargs.get("curves_to_normalize", None)
-    id_column: str = kwargs.get("id_column", None)
+    curves_to_normalize: Optional[List[str]] = kwargs.get("curves_to_normalize", None)
+    id_column: Optional[str] = kwargs.get("id_column", None)
     low_perc: float = kwargs.get("low_perc", 0.05)
     high_perc: float = kwargs.get("high_perc", 0.95)
-    user_key_wells: Dict[str, Dict[str, Union[str, float]]] = kwargs.get(
+    user_key_wells: Optional[Dict[str, Dict[str, Union[str, float]]]] = kwargs.get(
         "user_key_wells", None
     )
     save_key_wells: bool = kwargs.get("save_key_wells", False)
@@ -528,7 +665,7 @@ def normalize_curves(
                     "provided and no user_key_wells were provided!"
                 )
             else:
-                warnings.warn(
+                logger.warning(
                     "No id column was provided. Perfoming normalization "
                     "by assuming all the data is one well!",
                     stacklevel=2,
@@ -604,7 +741,7 @@ def normalize_curves(
 
         return (df, {"user_key_wells": key_wells})
     else:
-        warnings.warn(
+        logger.warning(
             "No curves were passed to curves_to_normalize so no normalization "
             "was performed!",
             stacklevel=2,
@@ -638,10 +775,10 @@ def scale_curves(
         tuple(pd.DataFrame, dict): scaled columns and the scaler object that was used to scale the
             scaled columns stored in a dict.
     """
-    curves_to_scale: List[str] = kwargs.get("curves_to_scale", None)
+    curves_to_scale: Optional[List[str]] = kwargs.get("curves_to_scale", None)
     scaler_method: str = kwargs.get("scaler_method", "RobustScaler")
     scaler_kwargs: Dict[str, Any] = kwargs.get("scaler_kwargs", {})
-    scaler: BaseEstimator = kwargs.get("scaler", None)
+    scaler: Optional[BaseEstimator] = kwargs.get("scaler", None)
     save_scaler: bool = kwargs.get("save_scaler", False)
     folder_path: str = kwargs.get("folder_path", "")
 
@@ -743,7 +880,7 @@ def process_wells(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
 
         return df
 
-    id_column: str = kwargs.get("id_column", None)
+    id_column: Optional[str] = kwargs.get("id_column", None)
     # Process per well if id_column exists otherwise process as one big set
     if id_column in df.columns:
         well_names = df[id_column].unique()
@@ -754,7 +891,7 @@ def process_wells(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
             res_df = pd.concat([res_df, well_df])
         df = res_df.copy()
     else:
-        warnings.warn(
+        logger.warning(
             "Not possible to process per well as well ID is not in dataset. "
             "Preprocessing was done considering all data is from the same well.",
             stacklevel=2,
@@ -786,14 +923,14 @@ def remove_noise(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
         pd.DataFrame: dataframe after removing noise
     """
     # Processing inputs
-    noisy_curves: List[str] = kwargs.get("noisy_curves", None)
+    noisy_curves: Optional[List[str]] = kwargs.get("noisy_curves", None)
     noise_removal_window = kwargs.get("noise_removal_window")
 
     if noisy_curves is None:
         # Only interested in numerical columns so no need to flood console with
         # warnings related to categorical curves
         noisy_curves, _ = utilities.get_col_types(df, warn=False)
-        warnings.warn(
+        logger.warning(
             "No noisy_curves were defined for the remove_noise function! "
             "Proceeding with removing noise in all detected numeric columns.",
             stacklevel=2,
@@ -806,7 +943,7 @@ def remove_noise(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
             .median()
         )
     else:
-        warnings.warn(
+        logger.warning(
             "No noise filtering was applied because one of the keyword arguments"
             " was not passed!",
             stacklevel=2,
@@ -852,6 +989,7 @@ def apply_calibration(
     for df, df_name in zip(
         [df_measured, df_predicted, calib_map],
         ["df_measured", "df_predicted", "calibration map"],
+        strict=False,
     ):
         missing_curves = [
             c
@@ -860,8 +998,7 @@ def apply_calibration(
         ]
         if len(missing_curves) > 0:
             raise ValueError(
-                f"Missing necessary curves in dataframe '{df_name}': "
-                f"{missing_curves}"
+                f"Missing necessary curves in dataframe '{df_name}': {missing_curves}"
             )
 
     # either get calibration from cdf if None or work on given map
@@ -871,15 +1008,9 @@ def apply_calibration(
 
     # standardize level names if either group or formation
     if standardize_level_names and level in ["FORMATION", "GROUP"]:
-        df_measured[level] = df_measured[level].apply(
-            utilities.standardize_group_formation_name
-        )
-        df_predicted[level] = df_predicted[level].apply(
-            utilities.standardize_group_formation_name
-        )
-        calib_map[level] = calib_map[level].apply(
-            utilities.standardize_group_formation_name
-        )
+        df_measured[level] = utilities.standardize_lsu_names(df_measured[level])
+        df_predicted[level] = utilities.standardize_lsu_names(df_predicted[level])
+        calib_map[level] = utilities.standardize_lsu_names(calib_map[level])
 
     calib_meas = utilities.get_calibration_values(
         df_measured,
@@ -904,7 +1035,7 @@ def apply_calibration(
 
     df_calibrated = df_predicted.copy()
     for well in df_predicted[id_column].unique():
-        print(well)
+        logger.info(well)
         df_m = df_predicted[df_predicted[id_column] == well]
         for c in curves:
             diffs = {
